@@ -32,6 +32,7 @@
 #include <QtCore/QHash>
 #include <QtCore/QVariant>
 #include <QtCore/QBuffer>
+#include <QTemporaryFile>
 #include <QUrl>
 
 #include <kprotocolinfo.h>
@@ -140,7 +141,7 @@ void JobTest::storedGet()
     KIO::StoredTransferJob *job = KIO::storedGet(u, KIO::NoReload, KIO::HideProgressInfo);
     QSignalSpy spyPercent(job, SIGNAL(percent(KJob*,ulong)));
     QVERIFY(spyPercent.isValid());
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     connect(job, SIGNAL(result(KJob*)),
             this, SLOT(slotGetResult(KJob*)));
     enterLoop();
@@ -165,7 +166,7 @@ void JobTest::put()
     QDateTime mtime = QDateTime::currentDateTime().addSecs(-30);   // 30 seconds ago
     mtime.setTime_t(mtime.toTime_t()); // hack for losing the milliseconds
     job->setModificationTime(mtime);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     connect(job, SIGNAL(result(KJob*)),
             this, SLOT(slotResult(KJob*)));
     connect(job, SIGNAL(dataReq(KIO::Job*,QByteArray&)),
@@ -215,7 +216,7 @@ void JobTest::storedPut()
     QDateTime mtime = QDateTime::currentDateTime().addSecs(-30);   // 30 seconds ago
     mtime.setTime_t(mtime.toTime_t()); // hack for losing the milliseconds
     job->setModificationTime(mtime);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     QVERIFY(job->exec());
     QFileInfo fileInfo(filePath);
     QVERIFY(fileInfo.exists());
@@ -237,7 +238,7 @@ void JobTest::storedPutIODevice()
     QDateTime mtime = QDateTime::currentDateTime().addSecs(-30);   // 30 seconds ago
     mtime.setTime_t(mtime.toTime_t()); // hack for losing the milliseconds
     job->setModificationTime(mtime);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     QVERIFY(job->exec());
     QFileInfo fileInfo(filePath);
     QVERIFY(fileInfo.exists());
@@ -269,6 +270,33 @@ void JobTest::storedPutIODeviceFile()
     QFile::remove(dest);
 }
 
+void JobTest::storedPutIODeviceTempFile()
+{
+    // Create a temp file in the current dir.
+    QTemporaryFile tempFile(QStringLiteral("jobtest-tmp"));
+    QVERIFY(tempFile.open());
+
+    // Write something into the file.
+    QTextStream stream(&tempFile);
+    stream << QStringLiteral("This is the put data");
+    stream.flush();
+    QVERIFY(QFileInfo(tempFile).size() > 0);
+
+    const QString dest = homeTmpDir() + QLatin1String("tmpfile-dest");
+    const QUrl destUrl = QUrl::fromLocalFile(dest);
+
+    // QTemporaryFiles are open in ReadWrite mode,
+    // so we don't need to close and reopen,
+    // but we need to rewind to the beginning.
+    tempFile.seek(0);
+    auto job = KIO::storedPut(&tempFile, destUrl, -1);
+
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
+    QVERIFY(QFileInfo::exists(dest));
+    QCOMPARE(QFileInfo(dest).size(), QFileInfo(tempFile).size());
+    QVERIFY(QFile::remove(dest));
+}
+
 void JobTest::storedPutIODeviceSlowDevice()
 {
     const QString filePath = homeTmpDir() + "fileFromHome";
@@ -284,7 +312,7 @@ void JobTest::storedPutIODeviceSlowDevice()
     mtime.setTime_t(mtime.toTime_t()); // hack for losing the milliseconds
     job->setModificationTime(mtime);
     job->setTotalSize(putDataContents.size());
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     job->setAsyncDataEnabled(true);
 
     QTimer t;
@@ -319,7 +347,7 @@ void JobTest::copyLocalFile(const QString &src, const QString &dest)
     const int perms = 0666;
     // copy the file with file_copy
     KIO::Job *job = KIO::file_copy(u, d, perms, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     bool ok = job->exec();
     QVERIFY(ok);
     QVERIFY(QFile::exists(dest));
@@ -345,8 +373,8 @@ void JobTest::copyLocalFile(const QString &src, const QString &dest)
     QFile::remove(dest);
     job = KIO::copy(u, d, KIO::HideProgressInfo);
     QSignalSpy spyCopyingDone(job, SIGNAL(copyingDone(KIO::Job*,QUrl,QUrl,QDateTime,bool,bool)));
-    job->setUiDelegate(0);
-    job->setUiDelegateExtension(0);
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr);
     ok = job->exec();
     QVERIFY(ok);
     QVERIFY(QFile::exists(dest));
@@ -368,11 +396,28 @@ void JobTest::copyLocalFile(const QString &src, const QString &dest)
     // cleanup and retry with KIO::copyAs()
     QFile::remove(dest);
     job = KIO::copyAs(u, d, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
-    job->setUiDelegateExtension(0);
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr);
     QVERIFY(job->exec());
     QVERIFY(QFile::exists(dest));
     QVERIFY(QFile::exists(src));     // still there
+
+    // Do it again, with Overwrite.
+    job = KIO::copyAs(u, d, KIO::Overwrite | KIO::HideProgressInfo);
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr);
+    QVERIFY(job->exec());
+    QVERIFY(QFile::exists(dest));
+    QVERIFY(QFile::exists(src));     // still there
+
+    // Do it again, without Overwrite (should fail).
+    job = KIO::copyAs(u, d, KIO::HideProgressInfo);
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr);
+    QVERIFY(!job->exec());
+
+    // Clean up
+    QFile::remove(dest);
 }
 
 void JobTest::copyLocalDirectory(const QString &src, const QString &_dest, int flags)
@@ -389,8 +434,8 @@ void JobTest::copyLocalDirectory(const QString &src, const QString &_dest, int f
     }
 
     KIO::Job *job = KIO::copy(u, d, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
-    job->setUiDelegateExtension(0);
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr);
     bool ok = job->exec();
     QVERIFY(ok);
     QVERIFY(QFile::exists(dest));
@@ -416,15 +461,15 @@ void JobTest::copyLocalDirectory(const QString &src, const QString &_dest, int f
     // Do it again, with Overwrite.
     // Use copyAs, we don't want a subdir inside d.
     job = KIO::copyAs(u, d, KIO::HideProgressInfo | KIO::Overwrite);
-    job->setUiDelegate(0);
-    job->setUiDelegateExtension(0);
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr);
     ok = job->exec();
     QVERIFY(ok);
 
     // Do it again, without Overwrite (should fail).
     job = KIO::copyAs(u, d, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
-    job->setUiDelegateExtension(0);
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr);
     ok = job->exec();
     QVERIFY(!ok);
 }
@@ -450,8 +495,8 @@ static void copyLocalSymlink(const QString &src, const QString &dest, const QStr
 
     // copy the symlink
     KIO::Job *job = KIO::copy(u, d, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
-    job->setUiDelegateExtension(0);
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr);
     QVERIFY2(job->exec(), qPrintable(job->error()));
     QVERIFY(QT_LSTAT(QFile::encodeName(dest).constData(), &buf) == 0); // dest exists
     QCOMPARE(linkTarget(dest), expectedLinkTarget);
@@ -553,21 +598,60 @@ void JobTest::copyFolderWithUnaccessibleSubfolder()
     KIO::CopyJob *job = KIO::copy(QUrl::fromLocalFile(src_dir), QUrl::fromLocalFile(dst_dir), KIO::HideProgressInfo);
 
     QSignalSpy spy(job, SIGNAL(warning(KJob*,QString,QString)));
-    job->setUiDelegate(0);   // no skip dialog, thanks
+    job->setUiDelegate(nullptr);   // no skip dialog, thanks
     QVERIFY(job->exec());
 
     QFile(inaccessible).setPermissions(QFile::Permissions(QFile::ReadOwner | QFile::WriteOwner | QFile::ExeOwner));
 
     KIO::DeleteJob *deljob1 = KIO::del(QUrl::fromLocalFile(src_dir), KIO::HideProgressInfo);
-    deljob1->setUiDelegate(0); // no skip dialog, thanks
+    deljob1->setUiDelegate(nullptr); // no skip dialog, thanks
     QVERIFY(deljob1->exec());
 
     KIO::DeleteJob *deljob2 = KIO::del(QUrl::fromLocalFile(dst_dir), KIO::HideProgressInfo);
-    deljob2->setUiDelegate(0); // no skip dialog, thanks
+    deljob2->setUiDelegate(nullptr); // no skip dialog, thanks
     QVERIFY(deljob2->exec());
 
     QCOMPARE(spy.count(), 1); // one warning should be emitted by the copy job
+}
 
+void JobTest::suspendFileCopy()
+{
+    const QString filePath = homeTmpDir() + "fileFromHome";
+    const QString dest = homeTmpDir() + "fileFromHome_copied";
+    createTestFile(filePath);
+
+    const QUrl u = QUrl::fromLocalFile(filePath);
+    const QUrl d = QUrl::fromLocalFile(dest);
+    KIO::Job *job = KIO::file_copy(u, d, KIO::HideProgressInfo);
+    QSignalSpy spyResult(job, SIGNAL(result(KJob*)));
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr);
+    QVERIFY(job->suspend());
+    QVERIFY(!spyResult.wait(300));
+    QVERIFY(job->resume());
+    QVERIFY(job->exec());
+    QVERIFY(QFile::exists(dest));
+    QFile::remove(dest);
+}
+
+void JobTest::suspendCopy()
+{
+    const QString filePath = homeTmpDir() + "fileFromHome";
+    const QString dest = homeTmpDir() + "fileFromHome_copied";
+    createTestFile(filePath);
+
+    const QUrl u = QUrl::fromLocalFile(filePath);
+    const QUrl d = QUrl::fromLocalFile(dest);
+    KIO::Job *job = KIO::copy(u, d, KIO::HideProgressInfo);
+    QSignalSpy spyResult(job, SIGNAL(result(KJob*)));
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr);
+    QVERIFY(job->suspend());
+    QVERIFY(!spyResult.wait(300));
+    QVERIFY(job->resume());
+    QVERIFY(job->exec());
+    QVERIFY(QFile::exists(dest));
+    QFile::remove(dest);
 }
 
 void JobTest::moveLocalFile(const QString &src, const QString &dest)
@@ -578,7 +662,7 @@ void JobTest::moveLocalFile(const QString &src, const QString &dest)
 
     // move the file with file_move
     KIO::Job *job = KIO::file_move(u, d, 0666, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     bool ok = job->exec();
     QVERIFY(ok);
     QVERIFY(QFile::exists(dest));
@@ -587,8 +671,8 @@ void JobTest::moveLocalFile(const QString &src, const QString &dest)
 
     // move it back with KIO::move()
     job = KIO::move(d, u, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
-    job->setUiDelegateExtension(0);
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr);
     ok = job->exec();
     QVERIFY(ok);
     QVERIFY(!QFile::exists(dest));
@@ -604,8 +688,8 @@ static void moveLocalSymlink(const QString &src, const QString &dest)
 
     // move the symlink with move, NOT with file_move
     KIO::Job *job = KIO::move(u, d, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
-    job->setUiDelegateExtension(0);
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr);
     bool ok = job->exec();
     if (!ok) {
         qWarning() << job->error();
@@ -616,8 +700,8 @@ static void moveLocalSymlink(const QString &src, const QString &dest)
 
     // move it back with KIO::move()
     job = KIO::move(d, u, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
-    job->setUiDelegateExtension(0);
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr);
     ok = job->exec();
     QVERIFY(ok);
     QVERIFY(QT_LSTAT(QFile::encodeName(dest).constData(), &buf) != 0); // doesn't exist anymore
@@ -637,8 +721,8 @@ void JobTest::moveLocalDirectory(const QString &src, const QString &dest)
     QUrl d = QUrl::fromLocalFile(dest);
 
     KIO::Job *job = KIO::move(u, d, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
-    job->setUiDelegateExtension(0);
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr);
     bool ok = job->exec();
     QVERIFY2(ok, qPrintable(job->errorString()));
     QVERIFY(QFile::exists(dest));
@@ -727,8 +811,8 @@ void JobTest::moveFileNoPermissions()
     // When trying to move it
     const QString dest = homeTmpDir() + "dest";
     KIO::CopyJob *job = KIO::move(QUrl::fromLocalFile(src), QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
-    job->setUiDelegate(0);
-    job->setUiDelegateExtension(0); // no skip dialog, thanks
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr); // no skip dialog, thanks
 
     // The job should fail with "access denied"
     QVERIFY(!job->exec());
@@ -758,8 +842,8 @@ void JobTest::moveDirectoryNoPermissions()
     // When trying to move it
     const QString dest = homeTmpDir() + "mdnp";
     KIO::CopyJob *job = KIO::move(QUrl::fromLocalFile(src), QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
-    job->setUiDelegate(0);
-    job->setUiDelegateExtension(0); // no skip dialog, thanks
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr); // no skip dialog, thanks
 
     // The job should fail with "access denied"
     QVERIFY(!job->exec());
@@ -784,7 +868,7 @@ void JobTest::listRecursive()
     QVERIFY(symlinkOk);
 #endif
     KIO::ListJob *job = KIO::listRecursive(QUrl::fromLocalFile(src), KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     connect(job, SIGNAL(entries(KIO::Job*,KIO::UDSEntryList)),
             SLOT(slotEntries(KIO::Job*,KIO::UDSEntryList)));
     bool ok = job->exec();
@@ -804,7 +888,7 @@ void JobTest::listRecursive()
 #ifndef Q_OS_WIN
                                       "dirFromHome_copied/testlink,dirFromHome_link,"
 #endif
-                                      "fileFromHome,fileFromHome_copied");
+                                      "fileFromHome");
 
     const QString joinedNames = m_names.join(QStringLiteral(","));
     if (joinedNames.toLatin1() != ref_names) {
@@ -819,14 +903,14 @@ void JobTest::listFile()
     const QString filePath = homeTmpDir() + "fileFromHome";
     createTestFile(filePath);
     KIO::ListJob *job = KIO::listDir(QUrl::fromLocalFile(filePath), KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     QVERIFY(!job->exec());
     QCOMPARE(job->error(), static_cast<int>(KIO::ERR_IS_FILE));
 
     // And list something that doesn't exist
     const QString path = homeTmpDir() + "fileFromHomeDoesNotExist";
     job = KIO::listDir(QUrl::fromLocalFile(path), KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     QVERIFY(!job->exec());
     QCOMPARE(job->error(), static_cast<int>(KIO::ERR_DOES_NOT_EXIST));
 }
@@ -837,10 +921,10 @@ void JobTest::killJob()
     KIO::ListJob *job = KIO::listDir(QUrl::fromLocalFile(src), KIO::HideProgressInfo);
     QVERIFY(job->isAutoDelete());
     QPointer<KIO::ListJob> ptr(job);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     qApp->processEvents(); // let the job start, it's no fun otherwise
     job->kill();
-    qApp->sendPostedEvents(0, QEvent::DeferredDelete); // process the deferred delete of the job
+    qApp->sendPostedEvents(nullptr, QEvent::DeferredDelete); // process the deferred delete of the job
     QVERIFY(ptr.isNull());
 }
 
@@ -850,9 +934,9 @@ void JobTest::killJobBeforeStart()
     KIO::Job *job = KIO::stat(QUrl::fromLocalFile(src), KIO::HideProgressInfo);
     QVERIFY(job->isAutoDelete());
     QPointer<KIO::Job> ptr(job);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     job->kill();
-    qApp->sendPostedEvents(0, QEvent::DeferredDelete); // process the deferred delete of the job
+    qApp->sendPostedEvents(nullptr, QEvent::DeferredDelete); // process the deferred delete of the job
     QVERIFY(ptr.isNull());
     qApp->processEvents(); // does KIO scheduler crash here? nope.
 }
@@ -862,7 +946,7 @@ void JobTest::deleteJobBeforeStart() // #163171
     const QString src = homeTmpDir();
     KIO::Job *job = KIO::stat(QUrl::fromLocalFile(src), KIO::HideProgressInfo);
     QVERIFY(job->isAutoDelete());
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     delete job;
     qApp->processEvents(); // does KIO scheduler crash here?
 }
@@ -874,7 +958,7 @@ void JobTest::directorySize()
     const QString src = homeTmpDir();
 
     KIO::DirectorySizeJob *job = KIO::directorySize(QUrl::fromLocalFile(src));
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     bool ok = job->exec();
     QVERIFY(ok);
     qDebug() << "totalSize: " << job->totalSize();
@@ -885,21 +969,21 @@ void JobTest::directorySize()
     QCOMPARE(job->totalSubdirs(), 3ULL); // see expected result in listRecursive() above
     QVERIFY(job->totalSize() > 54);
 #else
-    QCOMPARE(job->totalFiles(), 8ULL); // see expected result in listRecursive() above
+    QCOMPARE(job->totalFiles(), 7ULL); // see expected result in listRecursive() above
     QCOMPARE(job->totalSubdirs(), 4ULL); // see expected result in listRecursive() above
-    QVERIFY(job->totalSize() >= 325);
+    QVERIFY(job->totalSize() >= 260); // size of subdir entries is filesystem dependent. E.g. this is 16428 with ext4 but only 272 with xfs.
 #endif
 
-    qApp->sendPostedEvents(0, QEvent::DeferredDelete);
+    qApp->sendPostedEvents(nullptr, QEvent::DeferredDelete);
 }
 
 void JobTest::directorySizeError()
 {
     KIO::DirectorySizeJob *job = KIO::directorySize(QUrl::fromLocalFile(QStringLiteral("/I/Dont/Exist")));
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     bool ok = job->exec();
     QVERIFY(!ok);
-    qApp->sendPostedEvents(0, QEvent::DeferredDelete);
+    qApp->sendPostedEvents(nullptr, QEvent::DeferredDelete);
 }
 
 void JobTest::slotEntries(KIO::Job *, const KIO::UDSEntryList &lst)
@@ -1006,8 +1090,8 @@ void JobTest::getInvalidUrl()
     QVERIFY(!url.isValid());
 
     KIO::SimpleJob *job = KIO::get(url, KIO::NoReload, KIO::HideProgressInfo);
-    QVERIFY(job != 0);
-    job->setUiDelegate(0);
+    QVERIFY(job != nullptr);
+    job->setUiDelegate(nullptr);
 
     KIO::Scheduler::setJobPriority(job, 1); // shouldn't crash (#135456)
 
@@ -1017,16 +1101,16 @@ void JobTest::getInvalidUrl()
 
 void JobTest::slotMimetype(KIO::Job *job, const QString &type)
 {
-    QVERIFY(job != 0);
+    QVERIFY(job != nullptr);
     m_mimetype = type;
 }
 
 void JobTest::deleteFile()
 {
     const QString dest = otherTmpDir() + "fileFromHome_copied";
-    QVERIFY(QFile::exists(dest));
+    createTestFile(dest);
     KIO::Job *job = KIO::del(QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     bool ok = job->exec();
     QVERIFY(ok);
     QVERIFY(!QFile::exists(dest));
@@ -1053,7 +1137,7 @@ void JobTest::deleteDirectory()
 #endif
 
     KIO::Job *job = KIO::del(QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     bool ok = job->exec();
     QVERIFY(ok);
     QVERIFY(!QFile::exists(dest));
@@ -1076,7 +1160,7 @@ void JobTest::deleteSymlink(bool using_fast_path)
         QVERIFY(QFile::exists(dest));
     }
     KIO::Job *job = KIO::del(QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     bool ok = job->exec();
     QVERIFY(ok);
     QVERIFY(!QFile::exists(dest));
@@ -1109,7 +1193,7 @@ void JobTest::deleteManyDirs(bool using_fast_path)
     QTime dt;
     dt.start();
     KIO::Job *job = KIO::del(dirs, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     bool ok = job->exec();
     QVERIFY(ok);
     Q_FOREACH (const QUrl &dir, dirs) {
@@ -1158,7 +1242,7 @@ void JobTest::deleteManyFilesIndependently()
         QVERIFY(QFile::exists(file));
         //qDebug() << file;
         KIO::Job *job = KIO::del(url, KIO::HideProgressInfo);
-        job->setUiDelegate(0);
+        job->setUiDelegate(nullptr);
         bool ok = job->exec();
         QVERIFY(ok);
         QVERIFY(!QFile::exists(file));
@@ -1180,7 +1264,7 @@ void JobTest::deleteManyFilesTogether(bool using_fast_path)
 
     //qDebug() << file;
     KIO::Job *job = KIO::del(urls, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     bool ok = job->exec();
     QVERIFY(ok);
     qDebug() << "Deleted" << numFiles << "files in" << dt.elapsed() << "milliseconds";
@@ -1317,7 +1401,7 @@ void JobTest::chmodFile()
     QVERIFY(newPerm != origPerm);
     KFileItemList items; items << item;
     KIO::Job *job = KIO::chmod(items, newPerm, S_IWGRP /*TODO: QFile::WriteGroup*/, QString(), QString(), false, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     QVERIFY(job->exec());
 
     KFileItem newItem(QUrl::fromLocalFile(filePath));
@@ -1401,8 +1485,8 @@ void JobTest::moveFileDestAlreadyExists() // #157601
 
     QList<QUrl> urls; urls << QUrl::fromLocalFile(file1) << QUrl::fromLocalFile(file2);
     KIO::CopyJob *job = KIO::move(urls, QUrl::fromLocalFile(otherTmpDir()), KIO::HideProgressInfo);
-    job->setUiDelegate(0);
-    job->setUiDelegateExtension(0);
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr);
     job->setAutoSkip(true);
     bool ok = job->exec();
     QVERIFY(ok);
@@ -1462,8 +1546,8 @@ void JobTest::moveDestAlreadyExistsAutoRename(const QString &destDir, bool moveD
 
     QList<QUrl> urls; urls << QUrl::fromLocalFile(file1) << QUrl::fromLocalFile(file2);
     KIO::CopyJob *job = KIO::move(urls, QUrl::fromLocalFile(destDir), KIO::HideProgressInfo);
-    job->setUiDelegate(0);
-    job->setUiDelegateExtension(0);
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr);
     job->setAutoRename(true);
 
     //qDebug() << QDir(destDir).entryList();
@@ -1501,7 +1585,7 @@ void JobTest::moveAndOverwrite()
     createTestFile(existingDest);
 
     KIO::FileCopyJob *job = KIO::file_move(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(existingDest), -1, KIO::HideProgressInfo | KIO::Overwrite);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     bool ok = job->exec();
     QVERIFY(ok);
     QVERIFY(!QFile::exists(sourceFile)); // it was moved
@@ -1512,7 +1596,7 @@ void JobTest::moveAndOverwrite()
     createTestSymlink(existingDest, QFile::encodeName(sourceFile));
     QVERIFY(QFile::exists(existingDest));
     job = KIO::file_move(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(existingDest), -1, KIO::HideProgressInfo | KIO::Overwrite);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     ok = job->exec();
     QVERIFY(ok);
     QVERIFY(!QFile::exists(sourceFile)); // it was moved
@@ -1523,7 +1607,7 @@ void JobTest::moveAndOverwrite()
     createTestSymlink(existingDest, QFile::encodeName(sourceFile + "2"));
     QVERIFY(QFile::exists(existingDest));
     job = KIO::file_move(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(existingDest), -1, KIO::HideProgressInfo | KIO::Overwrite);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     ok = job->exec();
     QVERIFY(ok);
     QVERIFY(!QFile::exists(sourceFile)); // it was moved
@@ -1533,7 +1617,7 @@ void JobTest::moveAndOverwrite()
     createTestSymlink(existingDest);
     QVERIFY(!QFile::exists(existingDest)); // it exists, but it's broken...
     job = KIO::file_move(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(existingDest), -1, KIO::HideProgressInfo | KIO::Overwrite);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     ok = job->exec();
     QVERIFY(ok);
     QVERIFY(!QFile::exists(sourceFile)); // it was moved
@@ -1550,8 +1634,8 @@ void JobTest::moveOverSymlinkToSelf() // #169547
     QVERIFY(QFile::exists(existingDest));
 
     KIO::CopyJob *job = KIO::move(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(existingDest), KIO::HideProgressInfo);
-    job->setUiDelegate(0);
-    job->setUiDelegateExtension(0);
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr);
     bool ok = job->exec();
     QVERIFY(!ok);
     QCOMPARE(job->error(), (int)KIO::ERR_FILE_ALREADY_EXIST); // and not ERR_IDENTICAL_FILES!
@@ -1569,14 +1653,41 @@ void JobTest::createSymlink()
     const QString destDir = homeTmpDir() + "dest";
     QVERIFY(QDir().mkpath(destDir));
 
+    // With KIO::link (high-level)
     KIO::CopyJob *job = KIO::link(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(destDir), KIO::HideProgressInfo);
     QVERIFY(job->exec());
     QVERIFY(QFileInfo::exists(sourceFile));
     const QString dest = destDir + "/fileFromHome";
     QVERIFY(QFileInfo(dest).isSymLink());
+    QCOMPARE(QFileInfo(dest).symLinkTarget(), sourceFile);
+    QFile::remove(dest);
+
+    // With KIO::symlink (low-level)
+    const QString linkPath = destDir + "/link";
+    KIO::Job *symlinkJob = KIO::symlink(sourceFile, QUrl::fromLocalFile(linkPath), KIO::HideProgressInfo);
+    QVERIFY(symlinkJob->exec());
+    QVERIFY(QFileInfo::exists(sourceFile));
+    QVERIFY(QFileInfo(linkPath).isSymLink());
+    QCOMPARE(QFileInfo(linkPath).symLinkTarget(), sourceFile);
+
+    // Cleanup
     QVERIFY(QDir(destDir).removeRecursively());
 }
 
+void JobTest::createSymlinkTargetDirDoesntExist()
+{
+#ifdef Q_OS_WIN
+    QSKIP("Test skipped on Windows");
+#endif
+    const QString sourceFile = homeTmpDir() + "fileFromHome";
+    createTestFile(sourceFile);
+    const QString destDir = homeTmpDir() + "dest/does/not/exist";
+
+    KIO::CopyJob *job = KIO::link(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(destDir), KIO::HideProgressInfo);
+    QVERIFY(!job->exec());
+    QCOMPARE(job->error(), static_cast<int>(KIO::ERR_CANNOT_SYMLINK));
+}
+
 void JobTest::createSymlinkAsShouldSucceed()
 {
 #ifdef Q_OS_WIN
@@ -1594,7 +1705,7 @@ void JobTest::createSymlinkAsShouldSucceed()
     QVERIFY(QFile::remove(dest));
 }
 
-void JobTest::createSymlinkAsShouldFail()
+void JobTest::createSymlinkAsShouldFailDirectoryExists()
 {
 #ifdef Q_OS_WIN
     QSKIP("Test skipped on Windows");
@@ -1604,14 +1715,104 @@ void JobTest::createSymlinkAsShouldFail()
     const QString dest = homeTmpDir() + "dest";
     QVERIFY(QDir().mkpath(dest)); // dest exists as a directory
 
+    // With KIO::link (high-level)
     KIO::CopyJob *job = KIO::linkAs(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
     QVERIFY(!job->exec());
     QCOMPARE(job->error(), (int)KIO::ERR_DIR_ALREADY_EXIST);
     QVERIFY(QFileInfo::exists(sourceFile));
     QVERIFY(!QFileInfo::exists(dest + "/fileFromHome"));
+
+    // With KIO::symlink (low-level)
+    KIO::Job *symlinkJob = KIO::symlink(sourceFile, QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
+    QVERIFY(!symlinkJob->exec());
+    QCOMPARE(symlinkJob->error(), (int)KIO::ERR_DIR_ALREADY_EXIST);
+    QVERIFY(QFileInfo::exists(sourceFile));
+
+    // Cleanup
     QVERIFY(QDir().rmdir(dest));
 }
 
+void JobTest::createSymlinkAsShouldFailFileExists()
+{
+#ifdef Q_OS_WIN
+    QSKIP("Test skipped on Windows");
+#endif
+    const QString sourceFile = homeTmpDir() + "fileFromHome";
+    createTestFile(sourceFile);
+    const QString dest = homeTmpDir() + "testlink";
+    QFile::remove(dest); // just in case
+
+    // First time works
+    KIO::CopyJob *job = KIO::linkAs(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
+    QVERIFY(job->exec());
+    QVERIFY(QFileInfo(dest).isSymLink());
+
+    // Second time fails (already exists)
+    job = KIO::linkAs(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
+    QVERIFY(!job->exec());
+    QCOMPARE(job->error(), (int)KIO::ERR_FILE_ALREADY_EXIST);
+
+    // KIO::symlink fails too
+    KIO::Job *symlinkJob = KIO::symlink(sourceFile, QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
+    QVERIFY(!symlinkJob->exec());
+    QCOMPARE(symlinkJob->error(), (int)KIO::ERR_FILE_ALREADY_EXIST);
+
+    // Cleanup
+    QVERIFY(QFile::remove(sourceFile));
+    QVERIFY(QFile::remove(dest));
+}
+
+void JobTest::createSymlinkWithOverwriteShouldWork()
+{
+#ifdef Q_OS_WIN
+    QSKIP("Test skipped on Windows");
+#endif
+    const QString sourceFile = homeTmpDir() + "fileFromHome";
+    createTestFile(sourceFile);
+    const QString dest = homeTmpDir() + "testlink";
+    QFile::remove(dest); // just in case
+
+    // First time works
+    KIO::CopyJob *job = KIO::linkAs(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
+    QVERIFY(job->exec());
+    QVERIFY(QFileInfo(dest).isSymLink());
+
+    // Changing the link target, with overwrite, works
+    job = KIO::linkAs(QUrl::fromLocalFile(sourceFile + "2"), QUrl::fromLocalFile(dest), KIO::Overwrite | KIO::HideProgressInfo);
+    QVERIFY(job->exec());
+    QVERIFY(QFileInfo(dest).isSymLink());
+    QCOMPARE(QFileInfo(dest).symLinkTarget(), QString(sourceFile + "2"));
+
+    // Changing the link target using KIO::symlink, with overwrite, works
+    KIO::Job *symlinkJob = KIO::symlink(sourceFile + "3", QUrl::fromLocalFile(dest), KIO::Overwrite | KIO::HideProgressInfo);
+    QVERIFY(symlinkJob->exec());
+    QVERIFY(QFileInfo(dest).isSymLink());
+    QCOMPARE(QFileInfo(dest).symLinkTarget(), QString(sourceFile + "3"));
+
+    // Cleanup
+    QVERIFY(QFile::remove(dest));
+    QVERIFY(QFile::remove(sourceFile));
+}
+
+void JobTest::createBrokenSymlink()
+{
+#ifdef Q_OS_WIN
+    QSKIP("Test skipped on Windows");
+#endif
+    const QString sourceFile = "/does/not/exist";
+    const QString dest = homeTmpDir() + "testlink";
+    QFile::remove(dest); // just in case
+    KIO::CopyJob *job = KIO::linkAs(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
+    QVERIFY(job->exec());
+    QVERIFY(QFileInfo(dest).isSymLink());
+
+    // Second time fails (already exists)
+    job = KIO::linkAs(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
+    QVERIFY(!job->exec());
+    QCOMPARE(job->error(), (int)KIO::ERR_FILE_ALREADY_EXIST);
+    QVERIFY(QFile::remove(dest));
+}
+
 void JobTest::multiGet()
 {
     const int numFiles = 10;
@@ -1625,7 +1826,7 @@ void JobTest::multiGet()
     QSignalSpy spyMimeType(job, SIGNAL(mimetype(long,QString)));
     QSignalSpy spyResultId(job, SIGNAL(result(long)));
     QSignalSpy spyResult(job, SIGNAL(result(KJob*)));
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
 
     for (int i = 1; i < numFiles; ++i) {
         const QUrl url = urls.at(i);
