@@ -45,6 +45,7 @@
 #include <QDBusConnectionInterface>
 #include <QDBusInterface>
 #include <QDBusReply>
+#include <QHostInfo>
 
 #include <kiconloader.h>
 #include <kjobuidelegate.h>
@@ -85,8 +86,6 @@
 #include <kwindowsystem.h>
 #elif defined(Q_OS_WIN)
 #include <QDesktopServices>
-#include <qt_windows.h> // PPROCESS_INFORMATION
-#pragma message("TODO: remove #include <qt_windows.h> once we depend on Qt 5.3")
 #endif
 #include <qplatformdefs.h>
 #include <qstandardpaths.h>
@@ -166,9 +165,23 @@ void KRun::handleError(KJob *job)
     }
 }
 
-// This is called by foundMimeType, since it knows the mimetype of the URL
-bool KRun::runUrl(const QUrl &u, const QString &_mimetype, QWidget *window, bool tempFile, bool runExecutables, const QString &suggestedFileName, const QByteArray &asn)
+#ifndef KIOWIDGETS_NO_DEPRECATED
+bool KRun::runUrl(const QUrl &url, const QString &mimetype, QWidget *window, bool tempFile, bool runExecutables, const QString &suggestedFileName, const QByteArray &asn)
 {
+    RunFlags flags = tempFile ? KRun::DeleteTemporaryFiles : RunFlags();
+    if (runExecutables) {
+        flags |= KRun::RunExecutables;
+    }
+
+    return runUrl(url, mimetype, window, flags, suggestedFileName, asn);
+}
+#endif
+
+// This is called by foundMimeType, since it knows the mimetype of the URL
+bool KRun::runUrl(const QUrl &u, const QString &_mimetype, QWidget *window, RunFlags flags, const QString &suggestedFileName, const QByteArray &asn)
+{
+    const bool runExecutables = flags.testFlag(KRun::RunExecutables);
+    const bool tempFile = flags.testFlag(KRun::DeleteTemporaryFiles);
     bool noRun = false;
     bool noAuth = false;
     if (_mimetype == QLatin1String("inode/directory-locked")) {
@@ -503,7 +516,7 @@ class SecureMessageDialog : public QDialog
 {
     Q_OBJECT
 public:
-    SecureMessageDialog(QWidget *parent) : QDialog(parent), m_textEdit(0)
+    SecureMessageDialog(QWidget *parent) : QDialog(parent), m_textEdit(nullptr)
     {
     }
 
@@ -809,7 +822,7 @@ qint64 KRun::runService(const KService &_service, const QList<QUrl> &_urls, QWid
     }
 
     int i = KToolInvocation::startServiceByDesktopPath(
-                _service.entryPath(), QUrl::toStringList(urls), &error, 0L, &pid, myasn
+                _service.entryPath(), QUrl::toStringList(urls), &error, nullptr, &pid, myasn
             );
 
     if (i != 0) {
@@ -886,7 +899,7 @@ void KRun::KRunPrivate::init(const QUrl &url, QWidget *window,
     m_bAutoDelete = true;
     m_bProgressInfo = showProgressInfo;
     m_bFinished = false;
-    m_job = 0L;
+    m_job = nullptr;
     m_strURL = url;
     m_bScanFile = false;
     m_bIsDirectory = false;
@@ -930,7 +943,10 @@ void KRun::init()
         if (d->runExecutable(d->m_externalBrowser)) {
             return;
         }
-    } else if (d->m_strURL.isLocalFile()) {
+    } else if (d->m_strURL.isLocalFile() &&
+               (d->m_strURL.host().isEmpty() ||
+                (d->m_strURL.host() == QLatin1String("localhost")) ||
+                (d->m_strURL.host().compare(QHostInfo::localHostName(), Qt::CaseInsensitive) == 0))) {
         const QString localPath = d->m_strURL.toLocalFile();
         if (!QFile::exists(localPath)) {
             handleInitError(KIO::ERR_DOES_NOT_EXIST,
@@ -1180,7 +1196,7 @@ void KRun::slotTimeout()
 
 void KRun::slotStatResult(KJob *job)
 {
-    d->m_job = 0L;
+    d->m_job = nullptr;
     const int errCode = job->error();
     if (errCode) {
         // ERR_NO_CONTENT is not an error, but an indication no further
@@ -1241,12 +1257,12 @@ void KRun::slotScanMimeType(KIO::Job *, const QString &mimetype)
         qCWarning(KIO_WIDGETS) << "get() didn't emit a mimetype! Probably a kioslave bug, please check the implementation of" << url().scheme();
     }
     mimeTypeDetermined(mimetype);
-    d->m_job = 0;
+    d->m_job = nullptr;
 }
 
 void KRun::slotScanFinished(KJob *job)
 {
-    d->m_job = 0;
+    d->m_job = nullptr;
     const int errCode = job->error();
     if (errCode) {
         // ERR_NO_CONTENT is not an error, but an indication no further
@@ -1292,7 +1308,7 @@ void KRun::foundMimeType(const QString &type)
 
         job->putOnHold();
         KIO::Scheduler::publishSlaveOnHold();
-        d->m_job = 0;
+        d->m_job = nullptr;
     }
 
     Q_ASSERT(!d->m_bFinished);
@@ -1334,7 +1350,7 @@ void KRun::killJob()
     if (d->m_job) {
         //qDebug() << this << "m_job=" << d->m_job;
         d->m_job->kill();
-        d->m_job = 0L;
+        d->m_job = nullptr;
     }
 }
 
@@ -1587,7 +1603,7 @@ KProcessRunner::slotProcessExited(int exitCode, QProcess::ExitStatus exitStatus)
         // (or with a full path, if m_executable is absolute), and then in the PATH.
         if (!QFile(m_executable).exists() && QStandardPaths::findExecutable(m_executable).isEmpty()) {
             QEventLoopLocker locker;
-            KMessageBox::sorry(0L, i18n("Could not find the program '%1'", m_executable));
+            KMessageBox::sorry(nullptr, i18n("Could not find the program '%1'", m_executable));
         } else {
             //qDebug() << process->readAllStandardError();
         }
